{% load autocomplete_light_tags %}

<span class="autocomplete-light-text-widget {{ name }}" id="{{ widget.html_id }}-wrapper"
    {{ widget.widget_js_attributes|autocomplete_light_data_attributes }}
    {{ widget.autocomplete_js_attributes|autocomplete_light_data_attributes:'autocomplete-' }}
    >
    <input value="{{ value }}" type="text" name="{{ name }}" id="{{ widget.html_id }}" />
</span>